/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


package de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.roadcondition;

import de.tudarmstadt.maki.simonstrator.api.Time;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.EnvironmentProperty;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.RoadProperty;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.RoadNetworkEdge;

/**
 * @author Tobias Meuser (tobias.meuser@kom.tu-darmstadt.de)
 * @version 1.0 at 27.02.2018
 *
 */
public class RoadConditionProperty implements RoadProperty {

	private final Location _location;
	private final RoadNetworkEdge _edge;
	private long _detectionDate;
	private RoadCondition _roadCondition;

	public RoadConditionProperty(Location pLocation, RoadNetworkEdge pEdge) {
		this(pLocation, pEdge, RoadCondition.DRY);
	}

	public RoadConditionProperty(Location pLocation, RoadNetworkEdge pEdge, RoadCondition pRoadCondition) {
		_location = pLocation;
		_edge = pEdge;
		_detectionDate = Time.getCurrentTime();

		_roadCondition = pRoadCondition;
	}

	@Override
	public long getDetectionDate() {
		return _detectionDate;
	}

	@Override
	public Location getLocation() {
		return _location;
	}

	@Override
	public RoadNetworkEdge getEdge() {
		return _edge;
	}

	public RoadCondition getRoadCondition() {
		return _roadCondition;
	}

	public void setRoadCondition(RoadCondition pRoadCondition) {
		_roadCondition = pRoadCondition;
	}

	@Override
	public EnvironmentProperty getDefaultProperty() {
		return new RoadConditionProperty(_location, _edge, RoadCondition.DRY);
	}

}
