/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


package de.tudarmstadt.maki.simonstrator.api.component.vehicular.information;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import de.tudarmstadt.maki.simonstrator.api.component.pubsub.Notification;
import de.tudarmstadt.maki.simonstrator.api.component.pubsub.PubSubComponent;
import de.tudarmstadt.maki.simonstrator.api.component.pubsub.attribute.Attribute;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.LocationBasedEnvironmentProperty;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.RoadNetworkEdge;

/**
 * @author Tobias Meuser (tobias.meuser@kom.tu-darmstadt.de)
 * @version 1.0 at 05.03.2018
 *
 */
public class EnvironmentInformation<T extends LocationBasedEnvironmentProperty> implements PointInformation<T> {

	private T _environment;

	private Map<AvailableInformationAttributes, Object> _attributes = new HashMap<>();

	public EnvironmentInformation(T pEnvironment) {
		_environment = pEnvironment;
	}

	@Override
	public Location getLocation() {
		return _environment.getLocation();
	}

	@Override
	public long getDetectionDate() {
		return _environment.getDetectionDate();
	}

	@Override
	public <T> void setAttribute(AvailableInformationAttributes pKey, T pValue) {
		_attributes.put(pKey, pValue);
	}

	@SuppressWarnings("unchecked")
	@Override
	public <T> T getAttribute(AvailableInformationAttributes pKey) {
		if (AvailableInformationAttributes.DATE.equals(pKey)) {
			return (T)(Long) getDetectionDate();
		} else if (AvailableInformationAttributes.VALUE.equals(pKey)) {
			return (T) getValue();
		} else if (AvailableInformationAttributes.POSITION.equals(pKey)) {
			return (T) getLocation();
		}
		if (_attributes.containsKey(pKey)) {
			return (T) _attributes.get(pKey);
		}
		return null;
	}

	@Override
	public List<AvailableInformationAttributes> getAvailableAttributes() {
		return new ArrayList<>(_attributes.keySet());
	}

	@Override
	public <T> boolean hasAttribute(AvailableInformationAttributes pKey) {
		return _attributes.containsKey(pKey);
	}

	@Override
	public T getValue() {
		return _environment;
	}

	public static EnvironmentInformation<? extends LocationBasedEnvironmentProperty> createFromNotification(
			Notification pNotification) {
		Attribute<EnvironmentInformation> valueAttribute = pNotification
				.getAttribute(AvailableInformationAttributes.VALUE.getAttributeID(),
				EnvironmentInformation.class);
		if (valueAttribute != null) {
			EnvironmentInformation information = valueAttribute.getValue();
			Attribute<Object> ownerAttribute = pNotification
					.getAttribute(AvailableInformationAttributes.OWNER.getAttributeID(), Object.class);
			if (ownerAttribute != null) {
				information.setAttribute(AvailableInformationAttributes.OWNER, ownerAttribute.getValue());
			}

			return information;
		}
		return null;
	}

	public static <T extends Object> List<Attribute<?>> createAttributeList(
			EnvironmentInformation<? extends LocationBasedEnvironmentProperty> pInformation, PubSubComponent pPubSub) {
		List<Attribute<?>> attributes = new LinkedList<>();

		for (AvailableInformationAttributes attribute : pInformation.getAvailableAttributes()) {
			Class<T> expectedClass = (Class<T>) attribute.getExpectedClass();
			T value = pInformation.getAttribute(attribute);
			attributes.add(pPubSub.createAttribute(expectedClass, attribute.getAttributeID(), value));
		}

		attributes.add(
				pPubSub.createAttribute(RoadNetworkEdge.class, AvailableInformationAttributes.EDGE.getAttributeID(),
						pInformation.getAttribute(AvailableInformationAttributes.EDGE)));

		Attribute<EnvironmentInformation> ATTR_VALUE = pPubSub.createAttribute(EnvironmentInformation.class,
				AvailableInformationAttributes.VALUE.getAttributeID(), null);

		attributes.add(ATTR_VALUE.create(pInformation));

		return attributes;
	}

}
