package de.tudarmstadt.maki.simonstrator.api.util;

import java.util.List;
import java.util.stream.Collectors;

import de.tudarmstadt.maki.simonstrator.api.Oracle;
import de.tudarmstadt.maki.simonstrator.api.common.graph.INodeID;
import de.tudarmstadt.maki.simonstrator.api.component.ComponentNotAvailableException;
import de.tudarmstadt.maki.simonstrator.api.component.overlay.OverlayComponent;

/**
 * Node filtering methods.
 * 
 * @author Simon Luser
 *
 */
public final class Filter {

	/**
	 * @return a list of node-IDs that are online
	 */
	public static List<INodeID> getActiveNodes() {
		return Oracle.getAllHosts().stream().filter(host -> {
			try {
				return host.getComponent(OverlayComponent.class).isPresent();
			} catch (ComponentNotAvailableException e) {
				return false;
			}
		}).map(n -> n.getId()).collect(Collectors.toList());
	}

	/**
	 * 
	 * @return a list of all Node IDs
	 */
	public static List<INodeID> getAllNodeIDs() {
		return Oracle.getAllHosts().stream().map(node -> node.getId()).collect(Collectors.toList());
	}
}
