package de.tudarmstadt.maki.simonstrator.api.util.nodeselections;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Random;
import java.util.stream.Collectors;

import de.tudarmstadt.maki.simonstrator.api.Monitor;
import de.tudarmstadt.maki.simonstrator.api.Monitor.Level;
import de.tudarmstadt.maki.simonstrator.api.Randoms;
import de.tudarmstadt.maki.simonstrator.api.common.graph.INodeID;

/**
 * Determines initially random order of nodes ID's and tries to select the same
 * ones in the same order on consecutive calls. A node is skipped if it was
 * offline, to follow the distribution.
 * 
 * @author ConstiZ
 *
 */
public final class InitialRandomNodeSelection extends AbstractNodeSelection {

	private Random rnd;

	private List<INodeID> initialSelection;

	public InitialRandomNodeSelection() {
		rnd = Randoms.getRandom(InitialRandomNodeSelection.class);
	}

	@Override
	public void setAllNodeIDs(List<INodeID> allIDs) {
		this.allNodeIDs = allIDs;

		initialSelection = new ArrayList<INodeID>(allNodeIDs);
		Collections.shuffle(initialSelection, rnd);
		Monitor.log(InitialRandomNodeSelection.class, Level.DEBUG, "Initial node selection: " + initialSelection);
	}

	@Override
	public List<INodeID> selectNodes(int nodeCount, List<INodeID> allNodeIDs) throws IllegalSelectionException {
		validateSelection(nodeCount, allNodeIDs);

		// Select nodes from initialSelection, if they are in allNodes
		List<INodeID> selection = initialSelection.stream().filter(node -> allNodeIDs.contains(node))
				.collect(Collectors.toList());

		if (selection.size() < nodeCount)
			Monitor.log(InitialRandomNodeSelection.class, Level.ERROR, "Initial selection too short!");

		selection.subList(nodeCount, selection.size()).clear();
		return selection;
	}
}
